//#-hidden-code
//
//  Copyright © 2020 Apple Inc. All rights reserved.
//

import PlaygroundSupport
import Setup
import Canvas
import Scenes
import UIKit
import SpriteKit
import SwansQuest

PlaygroundPage.current.liveView = PlaygroundContext.liveViewController
PlaygroundContext.liveViewController.backgroundImage = #imageLiteral(resourceName: "LiveViewBackground.png")

Canvas.load(customFont: "Songkhla-Regular")

//#-end-hidden-code
//#-editable-code
// Touch "Run My Code" to continue your quest...

// The Lizard has decoded the sheet music, in case you need it:
 
let bass: [MIDINotes] = [
    .g3, .rest, .g2, .rest,
    .c3, .g3, .c4, .rest,
    .b4, .rest, .as4, .rest,
    .f2, .rest, .f3, .e3,
    .d3, .rest, .g2, .rest,
    .c4, .rest, .rest, .rest,
    .a3, .rest, .fs3, .rest,
    .f3, .d3, .c4, .a3
]

let treble: [MIDINotes] = [
    .rest, .b3, .c4, .d4,
    .e4, .rest, .rest, .d4,
    .e4, .a4, .g4, .e4,
    .d4, .c4, .a3, .rest,
    .rest, .c4, .e4, .f4,
    .g4, .rest, .rest, .a4,
    .g4, .e4, .c4, .e4,
    .d4, .rest, .rest, .rest
]

func performance(owner: Assessable) {
    let numberOfBeats = 32   // two bars of 4/4
    let duration = 16.0      // seconds

    // TODO: Build a step sequencer to play the Swan's song
    
    
    // Don't forget to call endPerformance() after you played through the song once.
    owner.endPerformance()
}

//#-end-editable-code
//#-hidden-code

let passages = [
    "Happy to see you again, traveler. I hope you did not have any trouble with the scroll?",
    
    "I’m really looking forward to your performance. As I told you earlier, this song is close to my heart, and it’s been too long since I’ve heard it.",
    
    "Okay, enough of my rambling. Are you ready to start?"
]
let script = Script(passages, success: "My dear traveler, that was such a wonderful performance.", tryAgain: "You’re so close! Please try again, traveler.")

let conversation = Conversation(with: .swan, using: script) {
    // Assessment: Check to see if they sequence the prescribed notes
    var expectedNoteSequence1: [MIDINotes] = [.g3, .rest, .g2, .rest, .c3, .g3, .c4, .rest, .b4, .rest, .as4, .rest, .f2, .rest, .f3, .e3, .d3, .rest, .g2, .rest, .c4, .rest, .rest, .rest, .a3, .rest, .fs3, .rest, .f3, .d3, .c4, .a3]
    var expectedNoteSequence2: [MIDINotes] = [.rest, .b3, .c4, .d4, .e4, .rest, .rest, .d4, .e4, .a4, .g4, .e4, .d4, .c4, .a3, .rest, .rest, .c4, .e4, .f4, .g4, .rest, .rest, .a4, .g4, .e4, .c4, .e4, .d4, .rest, .rest, .rest]
    
    var expectedUInts1 = expectedNoteSequence1.map { $0.midiCode }
    var expectedUInts2 = expectedNoteSequence2.map { $0.midiCode }

    var matchedSequences = 0
    for (kind, notes) in playedInstrumentNotes {
        if notes.starts(with: expectedUInts1)  || notes.starts(with: expectedUInts2) {
            matchedSequences += 1
        }
    }
    if matchedSequences > 1 {
        return true
    } else {
        return false
    }
}
conversation.performance = performance


let loading = Loading(with: .scrollChest)
loading.loadingDescription = "Turtle is walking..."

let hallComing = Hall(heroIs: .comingWithChest)
hallComing.loadingDescription = "The purple of the splendid hall reflects on the chest in the Turtle's hand, as they approach."

conversation.loadingDescription = "The swan speaks from her throne"

let presentingScript = Script([
    "Congratulations, brave Turtle. You have completed the final challenge. Your musical performances have warmed my heart and filled me with joy.",
    
    "To honor your accomplishment, and in recognition for your bravery and cleverness with code, I wish to bestow upon you...\n\nThe Order of the Golden Note!",
    
    "Wear this medal proudly on your travels. It will grant you passage anywhere in my kingdom, wherever your journey may take you next.",
    
    "Goodbye, and good luck."
])
let presentingTheAward = Conversation(with: .swan, using: presentingScript)

presentingTheAward.loadingDescription = "The Swan is smiling from beak to beak."

let victory = Victory()
victory.loadingDescription = "Congratulations, champion! You have completed Swan's Quest. We hope you enjoyed our four programming challenges, and will join us the next time a weary, traveler is in need of adventure. So long."

// Scene Order: HallEntrance, Hall coming, conversation with swan, Hall Holding note (TBD), congratulations
var runner = Runner([ loading, hallComing, conversation, presentingTheAward, victory ])


runner.start()

//#-end-hidden-code
